#!/usr/bin/env python3

"""
Generate src/bin/termptydbl.{c,h} from unicode files
used with ucd.all.flat.xml from
https://www.unicode.org/Public/UCD/latest/ucdxml/ucd.all.flat.zip
"""

import argparse
from collections import namedtuple
import xml.etree.ElementTree as ET

URange = namedtuple('unicode_range', ['width', 'start', 'end'])

def get_ranges(xmlfile, emoji_as_wide):
    tree = ET.parse(xmlfile)
    root = tree.getroot()
    repertoire = root.find("{http://www.unicode.org/ns/2003/ucd/1.0}repertoire")
    chars = repertoire.findall("{http://www.unicode.org/ns/2003/ucd/1.0}char")

    ranges = []
    r = URange('N', 0, 0)
    for c in chars:
        ea = c.get('ea')
        if ea in ('Na', 'H'):
            ea = 'N'
        if ea in ('F'):
            ea = 'W'
        cp = c.get('cp')
        if not cp:
            continue
        if emoji_as_wide:
            ext_pic = c.get('ExtPict')
            emoji = c.get('Emoji')
            if emoji == 'Y' and ext_pic == 'Y' and ea != 'A':
                ea = 'W'
            else:
                blk = c.get('blk')
                if blk == 'Misc_Pictographs':
                    ea = 'W'

        cp = int(cp, 16)
        if ea != r[0]:
            ranges.append(r)
            r = URange(ea, cp, cp)
        else:
            r = r._replace(end=cp)

    ranges.append(r)

    return ranges

def merge_ranges(ranges, is_same_width):
    res = []
    cur_range = ranges[0]
    for r in ranges:
        if is_same_width(r, cur_range):
            cur_range = cur_range._replace(end=r.end)
        else:
            res.append(cur_range)
            cur_range = r
    res.append(cur_range)
    return res

def skip_ranges(ranges, width_skipped):
    res = []
    for r in ranges:
        if r.width not in width_skipped:
            res.append(r)
    return res

def gen_header(cur_range, file_header):
    file_header.write(
"""/* XXX: Code generated by tool unicode_dbl_width.py */
#ifndef TERMINOLOGY_TERMPTY_DBL_H_
#define TERMINOLOGY_TERMPTY_DBL_H_ 1

Eina_Bool _termpty_is_wide(const Eina_Unicode g);
Eina_Bool _termpty_is_ambigous_wide(const Eina_Unicode g);

static inline Eina_Bool
_termpty_is_dblwidth_get(const Termpty *ty, const Eina_Unicode g)
{
   /* optimize for latin1 non-ambiguous */
""")
    file_header.write(f"   if (g <= 0x{cur_range.end:X})")
    file_header.write(
"""
     return EINA_FALSE;
   if (!ty->termstate.cjk_ambiguous_wide)
     return _termpty_is_wide(g);
   else
     return _termpty_is_ambigous_wide(g);
}

#endif
""")

def gen_ambigous(ranges, file_source):
    file_source.write(
"""
__attribute__((const))
Eina_Bool
_termpty_is_ambigous_wide(Eina_Unicode g)
{
    switch (g)
      {
""")
    def is_same_width(r1, r2):
        if r1.width == 'N':
            return r2.width == 'N'
        else:
            return r2.width in ('A', 'W')
    ranges = merge_ranges(ranges[1:], is_same_width)
    ranges = skip_ranges(ranges, ('N',))

    fallthrough = " EINA_FALLTHROUGH;"
    for idx, r in enumerate(ranges):
        if r.width == 'N':
            continue;
        if idx == len(ranges) -1:
            fallthrough = ""
        if r.start == r.end:
            file_source.write(f"       case 0x{r.start:X}:{fallthrough}\n")
        else:
            file_source.write(f"       case 0x{r.start:X} ... 0x{r.end:X}:{fallthrough}\n")

    file_source.write(
"""
        return EINA_TRUE;
    }
   return EINA_FALSE;
}
""")

def gen_wide(ranges, file_source):
    file_source.write(
"""
__attribute__((const))
Eina_Bool
_termpty_is_wide(Eina_Unicode g)
{
    switch (g)
      {
""")
    def is_same_width(r1, r2):
        if r1.width in ('N', 'A'):
            return r2.width in ('N', 'A')
        else:
            return r2.width == 'W'
    ranges = merge_ranges(ranges[1:], is_same_width)
    ranges = skip_ranges(ranges, ('N', 'A'))
    fallthrough = " EINA_FALLTHROUGH;"
    for idx, r in enumerate(ranges):
        if r.width in ('N', 'A'):
            continue;
        if idx == len(ranges) -1:
            fallthrough = ""
        if r.start == r.end:
            file_source.write(f"       case 0x{r.start:X}:{fallthrough}\n")
        else:
            file_source.write(f"       case 0x{r.start:X} ... 0x{r.end:X}:{fallthrough}\n")

    file_source.write(
"""
        return EINA_TRUE;
    }
   return EINA_FALSE;
}
""")


def gen_c(ranges, file_header, file_source):
    gen_header(ranges[0], file_header)
    file_source.write(
"""/* XXX: Code generated by tool unicode_dbl_width.py */
#include "private.h"

#include <Elementary.h>
#include "termpty.h"
#include "termptydbl.h"
""")
    gen_ambigous(ranges, file_source)
    gen_wide(ranges, file_source)

parser = argparse.ArgumentParser(description='Generate code handling different widths of unicode codepoints.')
parser.add_argument('xml', type=argparse.FileType('r'))
parser.add_argument('header', type=argparse.FileType('w'))
parser.add_argument('source', type=argparse.FileType('w'))

args = parser.parse_args()

ranges = get_ranges(args.xml, False)
gen_c(ranges, args.header, args.source)



#!/usr/bin/env python3

import argparse
import configparser

def ensure_premultiplied(t):
    (r, g, b, a) = t
    if a != 255 and (a > r or a > g or a > b):
        r = (r * a) // 255
        g = (g * a) // 255
        b = (b * a) // 255
    return (r, g, b, a)

def parse_color(color_string):
    h = color_string.lstrip('#')
    if len(h) == 6:
        return tuple(int(h[i:i+2], 16) for i in (0, 2, 4)) + (255,)
    elif len(h) == 3:
        return tuple(int(h[i]+h[i], 16) for i in (0, 1, 2)) + (255,)
    elif len(h) == 8:
        t = tuple(int(h[i:i+2], 16) for i in (0, 2, 4, 6))
        t = ensure_premultiplied(t)
        return t
    elif len(h) == 4:
        t = tuple(int(h[i]+h[i], 16) for i in (0, 1, 2, 3))
        t = ensure_premultiplied(t)
        return t

def write_color(out, color_string):
    (r, g, b, a) = parse_color(color_string)
    out.write('        group "Color" struct {\n')
    out.write('            value "r" uchar: {};\n'.format(r))
    out.write('            value "g" uchar: {};\n'.format(g))
    out.write('            value "b" uchar: {};\n'.format(b))
    out.write('            value "a" uchar: {};\n'.format(a))
    out.write('        }\n')

def write_md(out, cfg):
    out.write('    value "version" int: {};\n'
              .format(cfg.get('Main', 'version', fallback='1')))

    out.write('    value "md.version" int: {};\n'
              .format(cfg.get('Metadata', 'version', fallback='1')))
    out.write('    value "md.name" string: "{}";\n'
              .format(cfg['Metadata']['name']))
    out.write('    value "md.author" string: "{}";\n'
              .format(cfg['Metadata']['author']))
    out.write('    value "md.website" string: "{}";\n'
              .format(cfg.get('Metadata', 'website', fallback='')))
    out.write('    value "md.license" string: "{}";\n'
              .format(cfg['Metadata']['license']))

def write_named_color(out, cfg, section, color_name, default=None):
    out.write('    group "{}" struct {{\n'.format(color_name))
    write_color(out, cfg.get(section, color_name, fallback=default))
    out.write('    }\n')

def write_ui_colors(out, cfg):
    write_named_color(out, cfg, 'Colors', 'def', '#aaaaaa')
    write_named_color(out, cfg, 'Colors', 'bg', '#202020')
    write_named_color(out, cfg, 'Colors', 'fg', '#aaaaaa')
    write_named_color(out, cfg, 'Colors', 'main', '#3599ff')
    write_named_color(out, cfg, 'Colors', 'hl', '#ffffff')
    write_named_color(out, cfg, 'Colors', 'end_sel', '#ff3300')
    write_named_color(out, cfg, 'Colors', 'tab_missed_1', '#ff9933')
    write_named_color(out, cfg, 'Colors', 'tab_missed_2', '#ff3300')
    write_named_color(out, cfg, 'Colors', 'tab_missed_3', '#ff0000')
    write_named_color(out, cfg, 'Colors', 'tab_missed_over_1', '#ffff40')
    write_named_color(out, cfg, 'Colors', 'tab_missed_over_2', '#ff9933')
    write_named_color(out, cfg, 'Colors', 'tab_missed_over_3', '#ff0000')
    write_named_color(out, cfg, 'Colors', 'tab_title_2', '#000000')

def write_color_block(out, cfg, block):
    out.write('    group "{}" struct {{\n'.format(block))
    out.write('    group "Color_Block" struct {\n')
    write_named_color(out, cfg, block, 'def')
    write_named_color(out, cfg, block, 'black')
    write_named_color(out, cfg, block, 'red')
    write_named_color(out, cfg, block, 'green')
    write_named_color(out, cfg, block, 'yellow')
    write_named_color(out, cfg, block, 'blue')
    write_named_color(out, cfg, block, 'magenta')
    write_named_color(out, cfg, block, 'cyan')
    write_named_color(out, cfg, block, 'white')
    write_named_color(out, cfg, block, 'inverse_fg')
    write_named_color(out, cfg, block, 'inverse_bg')
    out.write('    }\n')
    out.write('    }\n')

def main():
    parser = argparse.ArgumentParser(description='Convert INI colorschemes to EET description files.')
    parser.add_argument('input_file',
                        type=argparse.FileType('r'),
                        help='INI File to convert')
    parser.add_argument('output_file',
                        type=argparse.FileType('w'),
                        help='EET description to write')
    args = parser.parse_args()

    cfg = configparser.ConfigParser()
    cfg.read_file(args.input_file)

    out = args.output_file

    assert(int(cfg['Main']['version']) == 1)
    out.write('group "Color_Scheme" struct {\n')
    write_md(out, cfg)
    write_ui_colors(out, cfg)
    write_color_block(out, cfg, 'Normal')
    write_color_block(out, cfg, 'Bright')
    write_color_block(out, cfg, 'Faint')
    out.write('}\n')

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import configparser
import sys
from ini2desc import parse_color

def blend_color(cfg, blend_factor, src, dest, color_name):
    (r1, g1, b1, a1) = parse_color(cfg.get('Colors', 'bg'))
    (r2, g2, b2, a2) = parse_color(cfg.get(src, color_name))
    def blend(c1, c2, f):
        d = c2 - c1
        return int(c1 + d * f)
    r = blend(r1, r2, blend_factor)
    g = blend(g1, g2, blend_factor)
    b = blend(b1, b2, blend_factor)
    a = blend(a1, a2, blend_factor)
    if a != 255:
        cfg.set(dest, color_name,
                '#{:02x}{:02x}{:02x}{:02x}'.format(r, g, b, a))
    else:
        cfg.set(dest, color_name,
                '#{:02x}{:02x}{:02x}'.format(r, g, b))

def main():
    parser = argparse.ArgumentParser(description='Generate Faint colors in INI colorschemes description files.')
    parser.add_argument('file',
                        type=argparse.FileType('r+'),
                        help='INI File to convert')
    parser.add_argument('blend_factor',
                        type=int, nargs='?', default=75,
                        help='blend factor between normal color and background')
    args = parser.parse_args()

    cfg = configparser.ConfigParser()
    cfg.read_file(args.file)

    f = args.blend_factor

    assert( 0 < f and f < 100)
    f = f / 100

    if not cfg.has_section('Faint'):
        cfg.add_section('Faint')
    if not cfg.has_section('BrightFaint'):
        cfg.add_section('BrightFaint')

    blend_color(cfg, f, 'Normal', 'Faint', 'def')
    blend_color(cfg, f, 'Normal', 'Faint', 'black')
    blend_color(cfg, f, 'Normal', 'Faint', 'red')
    blend_color(cfg, f, 'Normal', 'Faint', 'green')
    blend_color(cfg, f, 'Normal', 'Faint', 'yellow')
    blend_color(cfg, f, 'Normal', 'Faint', 'blue')
    blend_color(cfg, f, 'Normal', 'Faint', 'magenta')
    blend_color(cfg, f, 'Normal', 'Faint', 'cyan')
    blend_color(cfg, f, 'Normal', 'Faint', 'white')
    blend_color(cfg, f, 'Normal', 'Faint', 'inverse_fg')
    blend_color(cfg, f, 'Normal', 'Faint', 'inverse_bg')

    blend_color(cfg, f, 'Bright', 'BrightFaint', 'def')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'black')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'red')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'green')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'yellow')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'blue')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'magenta')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'cyan')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'white')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'inverse_fg')
    blend_color(cfg, f, 'Bright', 'BrightFaint', 'inverse_bg')

    args.file.seek(0)
    args.file.truncate()
    cfg.write(args.file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import configparser
import sys

def main():
    parser = argparse.ArgumentParser(description='Get color scheme name from an INI colorschemes description file.')
    parser.add_argument('input_file',
                        type=argparse.FileType('r'),
                        help='INI File to convert')
    args = parser.parse_args()

    cfg = configparser.ConfigParser()
    cfg.read_file(args.input_file)

    print(cfg.get('Metadata', 'name'))


if __name__ == "__main__":
    main()
